# -*- coding: utf-8 -*-

import os
import re
from .cache_utils import cpickle_cache, cached_property
current_dir = os.path.dirname(os.path.realpath(__file__))

class LazyData(object):

    @cached_property
    def en_us_dict(self):
        """
        werid example.
        >>> en_us_dict.check(u"跑")
        True
        """
        import enchant
        return enchant.Dict("en_US")

    @cached_property
    def two_length_words(self):
        """
        检查 2个字符 的字符组 是否是正常单词 或 词组
        e.g. `"At" in two_length_words`
        """
        def load__two_length_words__func():
            import nltk
            import marisa_trie
            return marisa_trie.Trie( \
                        sorted( \
                        list( \
                        set(\
                            [w1 for w1 in nltk.corpus.abc.words() \
                                if (len(w1) == 2) and \
                                    re.compile("[a-z]", re.IGNORECASE).match(w1[0]) \
                            ] \
                        ))))


        return cpickle_cache(current_dir + "/two_length_words.cPickle", load__two_length_words__func)

    @cached_property
    def regular_words(self):
        import nltk
        import marisa_trie
        from .regexp import word_regexp
        return marisa_trie.Trie([unicode(w1) for w1 in nltk.corpus.abc.words() if word_regexp.match(w1)])

    @cached_property
    def lemmatize(self):
        """
        return lemmatize method.

        必须得注明 词形 才行
        >>> lmtzr.lemmatize("having")
        'having'
        >>> lmtzr.lemmatize("having", 'v')
        'have'

        http://stackoverflow.com/questions/771918/how-do-i-do-word-stemming-or-lemmatization
        """
        from nltk.stem.wordnet import WordNetLemmatizer
        lmtzr = WordNetLemmatizer()

        def func(word1, tag1=None):
            if tag1 is None:
                word1 = lmtzr.lemmatize(word1, 'v')
                tag1 = 'n'
            return lmtzr.lemmatize(word1, tag1)
        return func

    @cached_property
    def tagged_words__dict(self):
        import nltk
        return {w1:t1.upper() for w1, t1 in nltk.corpus.brown.tagged_words()}



ld = LazyData()



# -*- coding: utf-8 -*-

from collections import Counter

def most_common_inspect(list1):
    new_list = []
    for s1 in list1:
        if not isinstance(s1, unicode):
            s1 = str(s1).decode("UTF-8")
        new_list.append(s1)

    cc = Counter(new_list).most_common()

    if len(cc) > 0:
        max_len = len(max([c1[0] for c1 in cc], key=lambda x1: len(x1))) + 5

        for c1 in cc:
            print c1[0].ljust(max_len, ' '), ' : ', c1[1]

    return cc


def uniq_seqs(seqs, uniq_lambda=None):
    if uniq_lambda is None: return list(set(seqs))

    __uniq = set([])
    __remove_idxes = []

    for idx1, seq1 in enumerate(seqs[:]):
       __id = uniq_lambda(seq1)
       if __id in __uniq:
           __remove_idxes.append(idx1)
       else:
           __uniq.add(__id)

    new_seqs = []
    for idx1, seq1 in enumerate(seqs[:]):
        if idx1 not in __remove_idxes:
            new_seqs.append(seq1)

    seqs = new_seqs

    return seqs



# -*- coding: utf-8 -*-

import os, sys, time, cPickle
from termcolor import cprint

from werkzeug.utils import cached_property

def cpickle_cache(filename, func1):
    """
    Usage:

    questions = cpickle_cache(pickle_filename, load_questions_func)
    """
    return cPickleCache(filename, func1).process()


class cPickleCache(object):
    def __init__(self, filename, func1):
        self.filename = filename
        self.func     = func1

    def process(self):
        tb = time.time()
        cprint(u"[%s] " % self.filename, end='')

        if os.path.isfile(self.filename):
            cprint(u"cache is already exists!", end='')
            result = cPickle.load(open(self.filename, 'rb'))
        else:
            cprint(u"generating cache ...", end='')
            result = self.func()
            cPickle.dump(result, open(self.filename, 'wb'))
        cprint(" load in %2.2f sec" % (time.time()-tb))
        return result



# -*- coding: utf-8 -*-

import os
import progressbar as pb


class ProcessNotifier(object):
    """
    Usage:

    for b1 in ProcessNotifier(Book).generator():
        # do b1
    """

    def __init__(self, scope, per1):
        """ TypeError: __init__() should return None, not 'generator' """
        if hasattr(scope, 'count') or isinstance(scope, list) or hasattr(scope, 'itervalues') or hasattr(scope, 'iterator'):
            self.scope = scope
        else:
            raise Exception(u"%s should be iteratable", str(scope))
        self.current_pid = os.getpid()
        self.per = per1

        # 兼容 list和dict
        self.scope_count = len(self.scope) if hasattr(self.scope, '__len__') else self.scope.count()


    def iterator(self):
        if 'iterator' in dir(self.scope)        : return self.scope.iterator()
        if 'itervalues' in dir(self.scope)      : return self.scope.itervalues()
        return self.scope

    def generator(self):
        """ Use a Generator to print the inner status when iterate a scope."""
        base_str = "[pid %i] To loading %i records " % (self.current_pid, self.scope_count)

        process_count = 0

        widgets = ['  ', base_str, pb.Percentage(), ' ', pb.Bar(),
                   ' ', ItemProcessSpeed()]

        if self.scope_count:
            self.pbar = pb.ProgressBar(widgets=widgets, maxval=self.scope_count).start()

        for record in self.iterator():
            process_count += 1
            if self.scope_count: self.pbar.update(process_count)
            yield record
        print


class ItemProcessSpeed(pb.ProgressBarWidget):
    def __init__(self):
        self.fmt = '%6.2f items/s'
    def update(self, pbar):
        if pbar.seconds_elapsed < 2e-6:#== 0:
            bps = 0.0
        else:
            bps = float(pbar.currval) / pbar.seconds_elapsed
        return self.fmt % bps


def process_notifier(scope, per1=1000):
    return ProcessNotifier(scope, per1).generator()

if __name__=='__main__':
    import time
    for i1 in process_notifier(range(500)):
        time.sleep(0.01)
